<?php
namespace Ak\View\Helper;

use RuntimeException;
use Laminas\View\Helper\AbstractHelper;
use Laminas\View\Helper\BasePath;
use Laminas\View\Helper\ServerUrl;

/**
 * Vraća url resursa definiranog u AssetManifestu.
 *
 * @package Ak\View\Helper
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/view
 */
class AssetUrl extends AbstractHelper
{
    /** @var string */
    private $manifestPath;
    /** @var string */
    private $assetBaseUrl;

    /** @var array */
    private $manifest;

    /** @var BasePath */
    private $basePathHelper;
    /** @var ServerUrl */
    private $serverUrlHelper;

    /**
     * Konstruktor.
     *
     * @param string $manifestPath
     * @param string $assetBaseUrl
     */
    public function __construct(string $manifestPath, string $assetBaseUrl)
    {
        $this->manifestPath = $manifestPath;
        $this->assetBaseUrl = $assetBaseUrl;
    }

    /**
     * @return BasePath
     */
    private function getBasePathHelper(): callable
    {
        if (null === $this->basePathHelper) {
            $this->basePathHelper = method_exists($this->view, 'plugin')
                ? $this->view->plugin('basePath')
                : new BasePath();
        }

        return $this->basePathHelper;
    }

    /**
     * @return callable
     */
    private function getServerUrlHelper(): callable
    {
        if (null === $this->serverUrlHelper) {
            $this->serverUrlHelper = method_exists($this->view, 'plugin')
                ? $this->view->plugin(ServerUrl::class)
                : new ServerUrl();
        }

        return $this->serverUrlHelper;
    }

    /**
     * @param string $asset
     *
     * @return string
     */
    public function __invoke(string $asset): string
    {
        $manifest = $this->getManifest();
        $basePath = $this->getBasePathHelper();
        $serverUrl = $this->getServerUrlHelper();

        if (!isset($manifest[$asset])) {
            throw new RuntimeException(sprintf("Asset s imenom '%s' ne postoji u manifestu.", $asset));
        }

        $assetUrl = sprintf('%s/%s', rtrim($this->assetBaseUrl, '/'), ltrim($manifest[$asset], '/'));
        return $serverUrl($basePath($assetUrl));
    }

    /**
     * Vraća asset manifest.
     *
     * @return array
     */
    protected function getManifest(): array
    {
        if (null === $this->manifest) {
            if (!file_exists($this->manifestPath) || !is_readable($this->manifestPath)) {
                throw new RuntimeException(sprintf(
                    "Datoteka '%s' ne postoji ili se ne može pročitati.", $this->manifestPath
                ));
            }

            $json = file_get_contents($this->manifestPath);
            $this->manifest = json_decode($json, true);

            if (null === $this->manifest) {
                throw new RuntimeException(sprintf(
                    "Došlo je do greške prilikom dekodiranje manifesta: %s.", json_last_error_msg()
                ));
            }
        }

        return $this->manifest;
    }
}
